/***********************策略对象**************************/
let strategies = {
  // 非空校验
  isNonEmpty: function (value: string, errorMsg: string) {
    if (value === '' || value === null || value === undefined) {
      return errorMsg;
    }
  },

  // 非0校验
  isNonZero: function (value: string, errorMsg: string) {
    if (value === "" || value === null || Number(value) === 0) {
      return errorMsg;
    }
  },

  // 最小长度校验
  minLength: function (value: string, length: string, errorMsg: string) {
    if (value.length < Number(length)) {
      return errorMsg;
    }
  },

  // 最大长度校验
  maxLength: function (value: string, length: string, errorMsg: string) {
    if (value.length > Number(length)) {
      return errorMsg;
    }
  },

  // 电话号码校验
  isMobile: function (value: string, errorMsg: string) {
    if (!/^1(3\d|4[5-9]|5[0-35-9]|6[2567]|7[0-8]|8\d|9[0-35-9])\d{8}$/.test(value)) {
      return errorMsg;
    }
  },

  // 数字、字母组合校验
  isCardNo: function (value: string, errorMsg: string) {
    let reg = /(^[a-za-z0-9]+$)/;
    if (!reg.test(value)) {
      return errorMsg;
    }
  },

  // 18位身份证校验
  isIDNo: function (value: string, errorMsg: string) {
    let reg = /(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X|x)$)/;
    if (!reg.test(value)) {
      return errorMsg;
    }
  },

  // 自定义校验
  isCustom: function (regExp: RegExp, value: string, errorMsg: string) {
    if (!regExp.test(value)) {
      return errorMsg;
    }
  }
};

/***********************Validator 类**************************/
let Validator = function () {
  this.cache = [];
};

Validator.prototype.add = function (value, rules) {
  var self = this;
  for (var i = 0, rule; rule = rules[i++];) {
    (function (rule) {
      var strategyAry = rule.strategy.split(':');
      var errorMsg = rule.errorMsg;
      var regExp = rule.regExp;
      self.cache.push(function () {
        var strategy = strategyAry.shift();
        strategyAry.unshift(value);
        strategyAry.push(errorMsg);
        if (regExp) {
          return strategies[strategy].apply(regExp, value, strategyAry);
        }
        return strategies[strategy].apply(value, strategyAry);
      });
    })(rule)
  }
};

Validator.prototype.start = function () {
  for (let i = 0, validatorFunc; validatorFunc = this.cache[i++];) {
    let errorMsg = validatorFunc();
    if (errorMsg) {
      return errorMsg;
    }
  }
};


/***********************客户调用代码**************************/
// import Validator from "./validator.ts"
function requiredVerify(item): String | null {
  let validator = new Validator();
  validator.add(item.name, [
    { strategy: "isNonEmpty", errorMsg: "请输入用户名" },
    { strategy: "minLength:6", errorMsg: "用户名长度不能小于6位" },
    { strategy: "maxLength:10", errorMsg: "用户名长度不能大于10位" },
    { strategy: "maxLength:10", errorMsg: "用户名长度不能大于10位" },
    { regExp: new RegExp(''), strategy: "isCustom", errorMsg: "这里是自定义校验的错误提示信息" },
  ]);

  let errorMsg = validator.start();
  return errorMsg;
}

export default Validator;